// dble_chk.c, Michael Dreesmann, michael@md-gis.de
//             problems with sscanf-function under cygwin with double vars
// ---------------------------------------------------------------------------
#include       <stdio.h>
#include       <stdlib.h>
#include       <string.h>
#include       <math.h>

#define        MAX_VAL        10

double         v[MAX_VAL] = {    6736.3,   78236.3,      2.5, 7380764.7, 77.9,
                              3370123.4, 5788123.0, 330123.0, 5780083.3, 22.9 };

int            write_test( double* sum );
int            read_test( double* sum );

int            main( int argc, char** argv )
{
  int            rc;
  double         sum_write = 0.0,
                 sum_read  = 0.0;

  if(  (rc=write_test( &sum_write )) != 0  )
  {
    printf( "error %d during write_test\n", rc );
    return(-1);
  }

  if(  (rc=read_test( &sum_read )) != 0  )
  {
    printf( "error %d during write_test\n", rc );
    return(-1);
  }

  printf( "sscanf - sum - write_test = %15.1f\n", sum_write );
  printf( "sscanf - sum - read_test  = %15.1f\n", sum_read );
  printf( "program termination correct\n" );

  return(0);
}


int            write_test( double* sum )
{
  int            rc = 0;
  FILE*          fp = fopen( "test.dat", "w" );
  int            i, j;

  *sum = 0.0;

  for( i=0; i<MAX_VAL; i++ )
  {
    for( j=0; j<=i; j++ )
      { fprintf( fp, "%.1f ", v[j] ); (*sum) += v[j]; }
    fprintf( fp, "\n" );
  }

  for( i=0; i<MAX_VAL; i++ )
  {
    fprintf( fp, "%d %*s", (i+4)*57, i+1, " " );
    for( j=0; j<=i; j++ )
      { fprintf( fp, "%.1f ", v[j] ); (*sum) += v[j]; }
    fprintf( fp, "\n" );
  }

  fclose( fp );

  return( rc );
}


int            read_test( double* sum )
{
  int            rc = 0;
  FILE*          fp = fopen( "test.dat", "r" );
  int            i, j;
  char           ln[256];
  char           first[20];
  double         a0, a1, a2, a3, a4, a5, a6, a7, a8, a9;

  *sum = 0.0;

  for( i=0; i<MAX_VAL; i++ )
  {
    if(  fgets( ln, 255, fp ) == NULL  )
      return(-2);
    switch(i)
    {
      case 0: sscanf( ln, "%lf", &a0 );
              (*sum) += a0;
              break;
      case 1: sscanf( ln, "%lf %lf", &a0, &a1 );
              (*sum) += a0;
              (*sum) += a1;
              break;
      case 2: sscanf( ln, "%lf %lf %lf", &a0, &a1, &a2 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              break;
      case 3: sscanf( ln, "%lf %lf %lf %lf", &a0, &a1, &a2, &a3 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              break;
      case 4: sscanf( ln, "%lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              break;
      case 5: sscanf( ln, "%lf %lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4, &a5 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              break;
      case 6: sscanf( ln, "%lf %lf %lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4, &a5, &a6 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              break;
      case 7: sscanf( ln, "%lf %lf %lf %lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              break;
      case 8: sscanf( ln, "%lf %lf %lf %lf %lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              (*sum) += a8;
              break;
      case 9: sscanf( ln, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8, &a9 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              (*sum) += a8;
              (*sum) += a9;
              break;
    }
  }

  for( i=0; i<MAX_VAL; i++ )
  {
    if(  fgets( ln, 255, fp ) == NULL  )
      return(-2);
    switch(i)
    {
      case 0: sscanf( ln, "%s %lf", first, &a0 );
              (*sum) += a0;
              break;
      case 1: sscanf( ln, "%s %lf %lf", first, &a0, &a1 );
              (*sum) += a0;
              (*sum) += a1;
              break;
      case 2: sscanf( ln, "%s %lf %lf %lf", first, &a0, &a1, &a2 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              break;
      case 3: sscanf( ln, "%s %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              break;
      case 4: sscanf( ln, "%s %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              break;
      case 5: sscanf( ln, "%s %lf %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4, &a5 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              break;
      case 6: sscanf( ln, "%s %lf %lf %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4, &a5, &a6 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              break;
      case 7: sscanf( ln, "%s %lf %lf %lf %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              break;
      case 8: sscanf( ln, "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              (*sum) += a8;
              break;
      case 9: sscanf( ln, "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", first, &a0, &a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8, &a9 );
              (*sum) += a0;
              (*sum) += a1;
              (*sum) += a2;
              (*sum) += a3;
              (*sum) += a4;
              (*sum) += a5;
              (*sum) += a6;
              (*sum) += a7;
              (*sum) += a8;
              (*sum) += a9;
              break;
    }
  }

  fclose( fp );

  return( rc );
}

// end
